/*
 * Copyright (C) 2024 CERN (www.cern.ch)
 * Author: Adam Wujek
 *
 * Released according to the GNU LGPL, version 2.1 or any later version.
 */

#include <ppsi/ppsi.h>
#include <ppsi/set-param.h>
#include <ppsi-wrs.h>


int set_param_delay_coefficient64(struct pp_instance *ppi,
				  int64_t delay_coefficient)
{
	if (!ppi->portDS)
		return -1;

	ppi->asymmetryCorrectionPortDS.scaledDelayCoefficient =
		delay_coefficient;
	ppi->portDS->delayAsymCoeff =
		pp_servo_calculateDelayAsymCoefficient(
			ppi->asymmetryCorrectionPortDS.scaledDelayCoefficient);

	return 0;
}

void set_param_ptpVersion(struct pp_instance *ppi)
{
	/* If cfg.versionNumber is set use it, if not set version based on
	 * the used extension */
	if (ppi->cfg.versionNumber) {
		ppi->portDS->versionNumber = ppi->cfg.versionNumber;
		ppi->portDS->minorVersionNumber = ppi->cfg.minorVersionNumber;
		pp_diag(ppi, config, 3, "PTP version for port wri%d is set "
			"manually to %d.%d\n",
			ppi->port_idx + 1, ppi->cfg.versionNumber,
			ppi->cfg.minorVersionNumber);
		return;
	}

	switch (ppi->protocol_extension) {
#if CONFIG_HAS_EXT_WR
	case PPSI_EXT_WR:
		ppi->portDS->versionNumber = PP_VERSION_PTP_WR;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP_WR;
		break;
#endif

#if CONFIG_HAS_EXT_L1SYNC
	case PPSI_EXT_L1S:
		ppi->portDS->versionNumber = PP_VERSION_PTP_HA;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP_HA;
		break;
#endif

	case PPSI_EXT_NONE:
		ppi->portDS->versionNumber = PP_VERSION_PTP;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP;
		break;

	default:
		break;
	}

	pp_diag(ppi, config, 3, "PTP version for port wri%d set to %d.%d based "
		"on the used extension\n",
		ppi->port_idx + 1, ppi->portDS->versionNumber,
		ppi->portDS->minorVersionNumber);

	return;
}

int set_param_delta_rxm64(struct pp_globals *ppg, int delta_rxm)
{
#if !CONFIG_HAS_EXT_WR
	/* If WR not included, return error */
	return -1;
#else
	int inst_i;
	wr_servo_ext_t *se;
	struct pp_instance *ppi;
	wrh_servo_t *s;
	int ret = -1;

	/* Set for all servos. When we support more servos probably this will
	 * have to change. */
	for (inst_i = 0; inst_i < ppg->nlinks; inst_i++) {
		ppi = INST(ppg, inst_i);
		if (!ppi)
			continue;

		if (ppi->state != PPS_SLAVE)
			continue;

		/* If not WR, continue */
		if (ppi->protocol_extension != PPSI_EXT_WR)
			continue;

		s = WRH_SRV(ppi);
		if (!s)
			continue;

		se = WRE_SRV(ppi);
		if (!se)
			continue;

		picos_to_pp_time(delta_rxm, &se->delta_rxm);

		ret = 0;
	}

	return ret;
#endif
}

int set_param_delta_txm64(struct pp_globals *ppg, int delta_txm)
{
#if !CONFIG_HAS_EXT_WR
	/* If WR not included, return error */
	return -1;
#else
	int inst_i;
	wr_servo_ext_t *se;
	struct pp_instance *ppi;
	wrh_servo_t *s;
	int ret = -1;

	/* Set for all servos. When we support more servos probably this will
	 * have to change. */
	for (inst_i = 0; inst_i < ppg->nlinks; inst_i++) {
		ppi = INST(ppg, inst_i);
		if (!ppi)
			continue;

		if (ppi->state != PPS_SLAVE)
			continue;

		/* If not WR, continue */
		if (ppi->protocol_extension != PPSI_EXT_WR)
			continue;

		s = WRH_SRV(ppi);
		if (!s)
			continue;

		se = WRE_SRV(ppi);
		if (!se)
			continue;

		picos_to_pp_time(delta_txm, &se->delta_txm);

		ret = 0;
	}

	return ret;
#endif
}

int set_param_egress_latency64(struct pp_instance *ppi, int64_t egress_latency)
{
	TimeInterval scaledBitSlide = 0;
	RelativeDifference scaledDelayCoefficient = 0;
	TimeInterval scaledSfpDeltaTx = 0;
	TimeInterval scaledSfpDeltaRx = 0;

	if (!ppi)
		return -1;

	ppi->timestampCorrectionPortDS.egressLatency
					= picos_to_interval(egress_latency);

	if (wrs_read_calibration_data(ppi,
				      &scaledBitSlide,
				      &scaledDelayCoefficient,
				      &scaledSfpDeltaTx,
				      &scaledSfpDeltaRx) != WRH_HW_CALIB_OK) {
		pp_diag(ppi, config, 1, "Cannot get calibration values "
			"bitslide, alpha, TX/Rx delays\n");
	}

	ppi->cfg.egressLatency_ps = egress_latency
				    - interval_to_picos(scaledSfpDeltaTx);

#if CONFIG_HAS_EXT_WR
	wr_servo_ext_t *se;
	wrh_servo_t *s;
	struct wr_dsport *wrp;
	FixedDelta *delta;

	/* If not WR, return */
	if (ppi->protocol_extension != PPSI_EXT_WR)
		return -1;

	wrp = WR_DSPOR(ppi);
	if (!wrp)
		return -1;

	s = WRH_SRV(ppi);
	if (!s)
		return -1;

	se = WRE_SRV(ppi);
	if (!se)
		return -1;

	delta = &wrp->deltaTx;
	interval_to_fixedDelta(ppi->timestampCorrectionPortDS.egressLatency,
			       delta);
	/* Update servo specific data */
	fixedDelta_to_pp_time(*delta, &se->delta_txs);
#endif

	return 0;
}

int set_param_ingress_latency64(struct pp_instance *ppi,
				int64_t ingress_latency)
{

	TimeInterval scaledBitSlide = 0;
	RelativeDifference scaledDelayCoefficient = 0;
	TimeInterval scaledSfpDeltaTx = 0;
	TimeInterval scaledSfpDeltaRx = 0;

	if (!ppi)
		return -1;

	ppi->timestampCorrectionPortDS.ingressLatency
					= picos_to_interval(ingress_latency);

	if (wrs_read_calibration_data(ppi,
				      &scaledBitSlide,
				      &scaledDelayCoefficient,
				      &scaledSfpDeltaTx,
				      &scaledSfpDeltaRx) != WRH_HW_CALIB_OK) {
		pp_diag(ppi, config, 1, "Cannot get calibration values "
			"bitslide, alpha, TX/Rx delays\n");
	}

	/* No need to adjust with scaledBitSlide */
	ppi->cfg.ingressLatency_ps = ingress_latency
				     - interval_to_picos(scaledSfpDeltaRx);

#if CONFIG_HAS_EXT_WR
	wr_servo_ext_t *se;
	wrh_servo_t *s;
	struct wr_dsport *wrp;
	FixedDelta *delta;

	/* If not WR, return */
	if (ppi->protocol_extension != PPSI_EXT_WR)
		return -1;

	wrp = WR_DSPOR(ppi);
	if (!wrp)
		return -1;

	s = WRH_SRV(ppi);
	if (!s)
		return -1;

	se = WRE_SRV(ppi);
	if (!se)
		return -1;

	delta = &wrp->deltaRx;
	interval_to_fixedDelta(ppi->timestampCorrectionPortDS.ingressLatency +
			       ppi->timestampCorrectionPortDS.semistaticLatency,
			       delta);
	/* Update servo specific data */
	fixedDelta_to_pp_time(*delta, &se->delta_rxs);
#endif

	return 0;
}

int set_param_constant_asymmetry64(struct pp_instance *ppi,
				   int64_t constantAsymmetry)
{
	ppi->asymmetryCorrectionPortDS.constantAsymmetry =
					picos_to_interval(constantAsymmetry);
	ppi->cfg.constantAsymmetry_ps = constantAsymmetry;

	return 0;
}

void set_param_externalPortConfiguration_desiredState(struct pp_instance *ppi,
						      int desiredState)
{
	ppi->externalPortConfigurationPortDS.desiredState = desiredState;
	ppi->cfg.desiredState = desiredState;
	/* The actual change of a state happens in state's handling */
}
